/*****************************************************************************************************************************
--
--    File Name    : alpha_object_read.v

--    Description  : This module implements Alpha blending control part which controls the foreground, background images to 
--                   alpha blending block                

-- Targeted device : Microsemi-SoC                     
-- Author          : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/
`timescale 1ns/1ps
module alpha_object_read #(
                           //*********************Parameters********************************//
                           parameter g_DDR_AXI_AWIDTH = 32, //Width of input data of channel1
                           parameter g_CHANNEL1_DATAWIDTH = 32, //Width of input data of V1 stream and Output data    
                           parameter g_CHANNEL1_X_Y_DATAWIDTH = 11 //Height and Width of V1 Stream   
                         )
                         (input wire SYS_CLK_I,
                          input wire RESET_n_I,
                          input wire start_i,
                          input wire next_object_line_read_i,
                          input wire frame_done_i,
                          input wire[(g_DDR_AXI_AWIDTH - 1) : 0] object_ddr_start_addr_i,
                          input wire[(g_CHANNEL1_X_Y_DATAWIDTH - 1) : 0] obj_w_i,
                          input wire[(g_CHANNEL1_X_Y_DATAWIDTH - 1) : 0] obj_h_i,
                          output wire read_req_o,
                          /*Read request to DDR arbiter*/
                          input wire read_ack_i,
                          /*Read request ack from DDR arbiter*/
                          input wire read_done_i,
                          /*Read done from DDR arbiter*/
                          output wire[(g_DDR_AXI_AWIDTH - 1) : 0] start_read_addr_o,
                          /*DDR address to read*/
                          output wire[15 : 0] bytes_to_read_o
                                                             /*Bytes to read from DDR*/
                                                             );
  parameter BYTES_TO_READ = g_CHANNEL1_DATAWIDTH / 8;
  parameter[3 : 0] ddr_read_idle = 4'd0,
                   sample_inputs = 4'd1,
                   transfer_calc = 4'd2,
                   ddr_read_req_gen = 4'd3,
                   wait_ddr_read_ack = 4'd4,
                   wait_ddr_read_done = 4'd5,
                   check_line_cntr = 4'd6,
                   wait_for_next_line_read_req = 4'd7,
                   wait_for_frame_done = 4'd8;
  reg[3 : 0] ddr_object_read_fsm;
  reg[(g_DDR_AXI_AWIDTH - 1) : 0] s_object_base_addr,
                                  s_ddr_address;
  reg[(g_CHANNEL1_X_Y_DATAWIDTH - 1) : 0] s_object_width;
  reg[(g_CHANNEL1_X_Y_DATAWIDTH - 1) : 0] s_object_height;
  reg[(g_CHANNEL1_X_Y_DATAWIDTH - 1) : 0] s_line_cntr;
  reg[(g_DDR_AXI_AWIDTH - 1) : 0] temp_reg;
  reg[15 : 0] s_bytes_to_read;
  reg s_read_req;
  reg s_start,
      s_start_d;
  wire start_pos_edge;
  reg s_next_object_line_read,
      s_next_object_line_read_d;
  wire s_next_object_line_read_pos_edge;
  reg s_frame_done,
      s_frame_done_d;
  wire s_frame_done_pos_edge;
  assign start_read_addr_o = s_ddr_address;
  assign bytes_to_read_o = s_bytes_to_read;
  assign read_req_o = s_read_req;
  /******************************************************************************************************************************/
  /********************************Process to generate the start input pulse ****************************************************/
  /******************************************************************************************************************************/
  always
    @(posedge SYS_CLK_I,
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          s_start <= 1'b0;
          s_start_d <= 1'b0;
        end
      else
        begin
          s_start <= start_i;
          s_start_d <= s_start;
        end
    end
  assign start_pos_edge = ~s_start_d & s_start;
  /******************************************************************************************************************************/
  /********************************Process to generate the next_object_line_read input pulse ************************************/
  /******************************************************************************************************************************/
  always
    @(posedge SYS_CLK_I,
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          s_next_object_line_read <= 1'b0;
          s_next_object_line_read_d <= 1'b0;
        end
      else
        begin
          s_next_object_line_read <= next_object_line_read_i;
          s_next_object_line_read_d <= s_next_object_line_read;
        end
    end
  assign s_next_object_line_read_pos_edge = ~s_next_object_line_read_d & s_next_object_line_read;
  /******************************************************************************************************************************/
  /********************************Process to generate the frame_done_i input pulse *********************************************/
  /******************************************************************************************************************************/
  always
    @(posedge SYS_CLK_I,
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          s_frame_done <= 1'b0;
          s_frame_done_d <= 1'b0;
        end
      else
        begin
          s_frame_done <= frame_done_i;
          s_frame_done_d <= s_frame_done;
        end
    end
  assign s_frame_done_pos_edge = ~s_frame_done_d & s_frame_done;
  /******************************************************************************************************************************/
  /********************************Main Control FSM *****************************************************************************/
  /******************************************************************************************************************************/
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          ddr_object_read_fsm <= ddr_read_idle;
          s_object_base_addr <= { g_DDR_AXI_AWIDTH { 1'b0 } };
          s_ddr_address <= { g_DDR_AXI_AWIDTH { 1'b0 } };
          s_object_width <= { g_CHANNEL1_X_Y_DATAWIDTH { 1'b0 } };
          s_object_height <= { g_CHANNEL1_X_Y_DATAWIDTH { 1'b0 } };
          s_line_cntr <= { g_CHANNEL1_X_Y_DATAWIDTH { 1'b0 } };
          s_bytes_to_read <= 16'd0;
          s_read_req <= 1'b0;
		  temp_reg <= { g_DDR_AXI_AWIDTH { 1'b0 } };
        end
      else
        begin
          case (ddr_object_read_fsm)
            ddr_read_idle:
              begin
                
                s_object_base_addr <= { g_DDR_AXI_AWIDTH { 1'b0 } };
                s_ddr_address <= { g_DDR_AXI_AWIDTH { 1'b0 } };
                s_object_width <= { g_CHANNEL1_X_Y_DATAWIDTH { 1'b0 } };
                s_object_height <= { g_CHANNEL1_X_Y_DATAWIDTH { 1'b0 } };
                s_line_cntr <= { g_CHANNEL1_X_Y_DATAWIDTH { 1'b0 } };
                s_bytes_to_read <= 16'd0;
                s_read_req <= 1'b0;
                if (start_pos_edge)
                  begin
                    ddr_object_read_fsm <= sample_inputs;
                  end
                else
                  begin
                    ddr_object_read_fsm <= ddr_read_idle;
                  end
              end
            sample_inputs:
              begin
                s_object_base_addr <= object_ddr_start_addr_i;
               // s_object_width <= obj_w_i;
                s_object_height <= obj_h_i;
                ddr_object_read_fsm <= transfer_calc;
				temp_reg <= (s_line_cntr * obj_w_i * BYTES_TO_READ);
              end
            transfer_calc:
              begin
                s_ddr_address <= s_object_base_addr + temp_reg;
                s_bytes_to_read <= s_object_width * BYTES_TO_READ;
                ddr_object_read_fsm <= ddr_read_req_gen;
              end
            ddr_read_req_gen:
              begin
                s_read_req <= 1'b1;
                ddr_object_read_fsm <= wait_ddr_read_ack;
              end
            wait_ddr_read_ack:
              begin
                if (read_ack_i)
                  begin
                    s_read_req <= 1'b0;
                    ddr_object_read_fsm <= wait_ddr_read_done;
                  end
                else
                  begin
                    s_read_req <= s_read_req;
                    ddr_object_read_fsm <= wait_ddr_read_ack;
                  end
              end
            wait_ddr_read_done:
              begin
                if (read_done_i)
                  begin
                    s_line_cntr <= s_line_cntr + 1'b1;
                    ddr_object_read_fsm <= check_line_cntr;
                  end
                else
                  begin
                    s_line_cntr <= s_line_cntr;
                    ddr_object_read_fsm <= wait_ddr_read_done;
                  end
              end
            check_line_cntr:
              begin
                if (s_line_cntr >= s_object_height)
                  begin
                    ddr_object_read_fsm <= wait_for_frame_done;
                  end
                else
                  begin
                    ddr_object_read_fsm <= wait_for_next_line_read_req;
                  end
              end
            wait_for_next_line_read_req:
              begin
                if (s_next_object_line_read_pos_edge)
                  begin
                    ddr_object_read_fsm <= transfer_calc;
                  end
                else
                  begin
                    ddr_object_read_fsm <= wait_for_next_line_read_req;
                  end
              end
            wait_for_frame_done:
              begin
                if (s_frame_done_pos_edge)
                  begin
                    ddr_object_read_fsm <= ddr_read_idle;
                  end
                else
                  begin
                    ddr_object_read_fsm <= wait_for_frame_done;
                  end
              end
            default:
              ddr_object_read_fsm <= ddr_read_idle;
          endcase
        end
    end
endmodule
